#ifndef API_DRIVER_DAEMON_H
#define API_DRIVER_DAEMON_H

#include "proxy.h"
#include "api-driver/structs.h"
#include <boost/thread/thread.hpp>

namespace api_driver {
    int64_t TimeNow();

    class TerminalApiDaemon {
        boost::thread daemon;

        void connectToApi();

        void run();

        std::shared_mutex stateMutex;
        obj::TerminalState state;
        obj::TerminalDeviceState stateDev;

        std::shared_mutex settingsMutex;
        obj::TerminalRxTxSettings settingsRxTx;
        #ifdef API_OBJECT_NETWORK_SETTINGS_ENABLE
        obj::TerminalNetworkSettings settingsNetwork;
        #endif
        #ifdef API_OBJECT_QOS_SETTINGS_ENABLE
        obj::TerminalQosSettings settingsQos;
        #endif

        std::shared_mutex firmwareMutex;
        obj::TerminalFirmwareVersion firmware;

    public:
        std::mutex cpApiMutex;
        proxy::CpProxy cp;

        #ifdef API_OBJECT_DEBUG_METRICS_ENABLE
        obj::StatisticsLogger statsLogs;
        #endif

        explicit TerminalApiDaemon();

        /**
        * Получение статистики, копирует текущие значения в структуры, переданные по указателю. Если передан пустой указатель, копирования не произойдет.
        */
        void getState(obj::TerminalState &dest);

        void getDeviceState(obj::TerminalDeviceState &dest);

        // /**
        //  * Получение настроек, копирует текущие значения в структуры, переданные по указателю. Если передан пустой указатель, копирования не произойдет.
        //  */
        void getSettingsRxTx(obj::TerminalRxTxSettings &dest);

        void getNetworkSettings(obj::TerminalNetworkSettings &dest);

        void getQosSettings(bool &isEnabled, std::string &json);

        void setSettingsRxTx(obj::TerminalRxTxSettings &s, bool readback = true);

        void setSettingsDpdi(DPDI_parmeters &s, bool readback = true);

        void setSettingsBucLnb(buc_lnb_settings &bucLnb, bool readback = true);

        void setQosSettings(bool enabled, const std::string &str, bool readback = true);

        void setNetworkSettings(obj::TerminalNetworkSettings &s, bool readback = true);

        obj::TerminalFirmwareVersion getFirmware();

        void resetPacketStatistics();

        void resetDefaultSettings();

        ~TerminalApiDaemon();
    };
}


#endif //API_DRIVER_DAEMON_H
